@@ -1,3 +1,5 @@
+//! The `ControlDir` class provides a high-level interface to control directories,
+//! e.g. ".bzr" or ".git" directories.
 use crate::branch::{py_tag_selector, Branch, RegularBranch};
 use crate::error::Error;
 use crate::repository::Repository;
@@ -68,7 +70,7 @@ impl ControlDir {
 
     #[deprecated]
     pub fn create_branch_convenience(base: &url::Url) -> Result<Box<dyn Branch>, Error> {
-        create_branch_convenience(base)
+        create_branch_convenience(base, None)
     }
 
     #[deprecated]
@@ -370,7 +372,11 @@ pub fn open_tree_or_branch(
 #[test]
 fn test_open_tree_or_branch() {
     let tmp_dir = tempfile::tempdir().unwrap();
-    create_branch_convenience(&url::Url::from_directory_path(tmp_dir.path()).unwrap()).unwrap();
+    create_branch_convenience(
+        &url::Url::from_directory_path(tmp_dir.path()).unwrap(),
+        None,
+    )
+    .unwrap();
     let (wt, branch) = open_tree_or_branch(
         &url::Url::from_directory_path(tmp_dir.path()).unwrap(),
         None,
@@ -597,11 +603,22 @@ impl AsFormat for &ControlDirFormat {
     }
 }
 
-pub fn create_branch_convenience(base: &url::Url) -> Result<Box<dyn Branch>, Error> {
+pub fn create_branch_convenience(
+    base: &url::Url,
+    force_new_tree: Option<bool>,
+) -> Result<Box<dyn Branch>, Error> {
     Python::with_gil(|py| {
         let m = py.import_bound("breezy.controldir")?;
         let cd = m.getattr("ControlDir")?;
-        let branch = cd.call_method("create_branch_convenience", (base.to_string(),), None)?;
+        let kwargs = PyDict::new_bound(py);
+        if let Some(force_new_tree) = force_new_tree {
+            kwargs.set_item("force_new_tree", force_new_tree)?;
+        }
+        let branch = cd.call_method(
+            "create_branch_convenience",
+            (base.to_string(),),
+            Some(&kwargs),
+        )?;
         Ok(Box::new(RegularBranch::new(branch.to_object(py))) as Box<dyn Branch>)
     })
 }
@@ -643,8 +660,11 @@ fn test_create_standalone_workingtree() {
 #[test]
 fn test_create_branch_convenience() {
     let tmp_dir = tempfile::tempdir().unwrap();
-    let branch =
-        create_branch_convenience(&url::Url::from_directory_path(tmp_dir.path()).unwrap()).unwrap();
+    let branch = create_branch_convenience(
+        &url::Url::from_directory_path(tmp_dir.path()).unwrap(),
+        None,
+    )
+    .unwrap();
 
     assert_eq!(
         branch.get_user_url(),
@@ -663,7 +683,9 @@ pub fn all_probers() -> Vec<Box<dyn Prober>> {
     Python::with_gil(|py| -> PyResult<Vec<Box<dyn Prober>>> {
         let m = py.import_bound("breezy.controldir")?;
         let cdf = m.getattr("ControlDirFormat")?;
-        let probers = cdf.call_method0("all_probers")?.extract::<Vec<PyObject>>()?;
+        let probers = cdf
+            .call_method0("all_probers")?
+            .extract::<Vec<PyObject>>()?;
         Ok(probers
             .into_iter()
             .map(|p| Box::new(PyProber(p)) as Box<dyn Prober>)
