@@ -1,3 +1,4 @@
+//! Error handling for the Breezy Python bindings
 use pyo3::import_exception;
 use pyo3::prelude::*;
 use pyo3::PyErr;
@@ -20,6 +21,7 @@ import_exception!(http.client, IncompleteRead);
 import_exception!(breezy.bzr, LineEndingError);
 import_exception!(breezy.errors, InvalidHttpResponse);
 import_exception!(breezy.errors, AlreadyControlDirError);
+import_exception!(breezy.errors, AlreadyBranchError);
 import_exception!(breezy.errors, DivergedBranches);
 import_exception!(breezy.workspace, WorkspaceDirty);
 import_exception!(breezy.transport, NoSuchFile);
@@ -51,6 +53,15 @@ import_exception!(breezy.plugins.gitlab.forge, GitLabConflict);
 import_exception!(breezy.plugins.gitlab.forge, ProjectCreationTimeout);
 import_exception!(breezy.forge, SourceNotDerivedFromTarget);
 
+lazy_static::lazy_static! {
+    // Only present in breezy << 4.0
+    pub static ref BreezyConnectionError: Option<PyObject> = { Python::with_gil(|py| {
+        let m = py.import_bound("breezy.errors").unwrap();
+        m.getattr("ConnectionError").ok().map(|x| x.to_object(py))
+    })
+};
+}
+
 #[derive(Debug)]
 pub enum Error {
     Other(PyErr),
@@ -76,6 +87,7 @@ pub enum Error {
         std::collections::HashMap<String, String>,
     ),
     AlreadyControlDir(std::path::PathBuf),
+    AlreadyBranch(std::path::PathBuf),
     DivergedBranches,
     WorkspaceDirty(std::path::PathBuf),
     NoSuchFile(std::path::PathBuf),
@@ -185,6 +197,7 @@ impl std::fmt::Display for Error {
                 }
             }
             Self::AlreadyControlDir(p) => write!(f, "Already exists: {}", p.display()),
+            Self::AlreadyBranch(p) => write!(f, "Already a branch: {}", p.display()),
             Self::DivergedBranches => write!(f, "Diverged branches"),
             Self::WorkspaceDirty(p) => write!(f, "Workspace dirty at {}", p.display()),
             Self::NoSuchFile(p) => write!(f, "No such file: {}", p.to_string_lossy()),
@@ -308,7 +321,7 @@ impl From<PyErr> for Error {
                     value.getattr("extra").unwrap().extract().unwrap(),
                 )
             } else if err.is_instance_of::<pyo3::exceptions::PyConnectionError>(py) {
-                Error::ConnectionError(value.getattr("message").unwrap().extract().unwrap())
+                Error::ConnectionError(err.to_string())
             } else if err.is_instance_of::<UnsupportedFormatError>(py) {
                 Error::UnsupportedFormat(value.getattr("format").unwrap().extract().unwrap())
             } else if err.is_instance_of::<UnsupportedVcs>(py) {
@@ -322,15 +335,10 @@ impl From<PyErr> for Error {
                 )
             } else if err.is_instance_of::<LineEndingError>(py) {
                 Error::LineEndingError(value.getattr("file").unwrap().extract().unwrap())
-            } else if err.is_instance_of::<InvalidHttpResponse>(py) {
-                Error::InvalidHttpResponse(
-                    value.getattr("path").unwrap().extract().unwrap(),
-                    value.getattr("msg").unwrap().extract().unwrap(),
-                    value.getattr("orig_error").unwrap().extract().unwrap(),
-                    value.getattr("headers").unwrap().extract().unwrap(),
-                )
             } else if err.is_instance_of::<AlreadyControlDirError>(py) {
                 Error::AlreadyControlDir(value.getattr("path").unwrap().extract().unwrap())
+            } else if err.is_instance_of::<AlreadyBranchError>(py) {
+                Error::AlreadyBranch(value.getattr("path").unwrap().extract().unwrap())
             } else if err.is_instance_of::<DivergedBranches>(py) {
                 Error::DivergedBranches
             } else if err.is_instance_of::<WorkspaceDirty>(py) {
@@ -525,9 +533,24 @@ impl From<PyErr> for Error {
                 Error::GitLabConflict(value.getattr("reason").unwrap().extract().unwrap())
             } else if err.is_instance_of::<SourceNotDerivedFromTarget>(py) {
                 Error::SourceNotDerivedFromTarget
+            } else if BreezyConnectionError
+                .as_ref()
+                .and_then(|cls| {
+                    Python::with_gil(|py| Some(err.is_instance_bound(py, cls.bind(py))))
+                })
+                .unwrap_or(false)
+            {
+                Error::ConnectionError(err.to_string())
             // Intentionally sorted below the more specific errors
+            } else if err.is_instance_of::<InvalidHttpResponse>(py) {
+                Error::InvalidHttpResponse(
+                    value.getattr("path").unwrap().extract().unwrap(),
+                    value.getattr("msg").unwrap().extract().unwrap(),
+                    value.getattr("orig_error").unwrap().extract().unwrap(),
+                    value.getattr("headers").unwrap().extract().unwrap(),
+                )
             } else if err.is_instance_of::<TransportError>(py) {
-                Error::TransportError(value.getattr("message").unwrap().extract().unwrap())
+                Error::TransportError(value.getattr("msg").unwrap().extract().unwrap())
             } else {
                 Self::Other(err)
             }
@@ -569,6 +592,9 @@ impl From<Error> for PyErr {
             Error::AlreadyControlDir(path) => {
                 AlreadyControlDirError::new_err((path.to_string_lossy().to_string(),))
             }
+            Error::AlreadyBranch(path) => {
+                AlreadyBranchError::new_err((path.to_string_lossy().to_string(),))
+            }
             Error::DivergedBranches => {
                 Python::with_gil(|py| DivergedBranches::new_err((py.None(), py.None())))
             }
@@ -1166,3 +1192,13 @@ fn test_project_creation_timeout() {
         assert!(p.is_instance_of::<ProjectCreationTimeout>(py), "{}", p);
     });
 }
+
+#[test]
+fn test_already_branch() {
+    let e = Error::AlreadyBranch(std::path::PathBuf::from("foo"));
+    let p: PyErr = e.into();
+    // Verify that p is an instance of AlreadyBranchError
+    Python::with_gil(|py| {
+        assert!(p.is_instance_of::<AlreadyBranchError>(py), "{}", p);
+    });
+}
