@@ -17,6 +17,7 @@
 
 pub mod bazaar;
 pub mod branch;
+pub mod commit;
 pub mod config;
 pub mod controldir;
 pub mod cvs;
@@ -58,6 +59,7 @@ pub mod transport;
 pub mod tree;
 pub mod urlutils;
 pub mod version;
+pub mod workingtree;
 pub mod workspace;
 
 #[cfg(feature = "debian")]
