@@ -1,5 +1,5 @@
-use crate::branch::{Branch, RegularBranch};
-use crate::controldir::ControlDir;
+//! Trees
+use crate::branch::Branch;
 use crate::error::Error;
 use crate::lock::Lock;
 use crate::revisionid::RevisionId;
@@ -8,7 +8,7 @@ use pyo3::prelude::*;
 pub type Path = std::path::Path;
 pub type PathBuf = std::path::PathBuf;
 
-#[derive(Debug, PartialEq)]
+#[derive(Debug, PartialEq, Clone, Eq)]
 pub enum Kind {
     File,
     Directory,
@@ -25,13 +25,31 @@ impl Kind {
             Kind::TreeReference => "+",
         }
     }
+}
 
-    pub fn to_string(&self) -> &'static str {
+impl std::fmt::Display for Kind {
+    fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
         match self {
-            Kind::File => "file",
-            Kind::Directory => "directory",
-            Kind::Symlink => "symlink",
-            Kind::TreeReference => "tree-reference",
+            Kind::File => write!(f, "file"),
+            Kind::Directory => write!(f, "directory"),
+            Kind::Symlink => write!(f, "symlink"),
+            Kind::TreeReference => write!(f, "tree-reference"),
+        }
+    }
+}
+
+impl std::str::FromStr for Kind {
+    type Err = String;
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        match s {
+            "file" => Ok(Kind::File),
+            "directory" => Ok(Kind::Directory),
+            "symlink" => Ok(Kind::Symlink),
+            "tree-reference" => Ok(Kind::TreeReference),
+            n => {
+                return Err(format!("Invalid kind: {}", n));
+            }
         }
     }
 }
@@ -406,6 +424,18 @@ pub trait Tree: ToPyObject {
 }
 
 pub trait MutableTree: Tree {
+    fn add(&self, files: &[&Path]) -> Result<(), Error> {
+        Python::with_gil(|py| -> Result<(), PyErr> {
+            self.to_object(py).call_method1(
+                py,
+                "add",
+                (files.iter().map(|p| p.to_path_buf()).collect::<Vec<_>>(),),
+            )?;
+            Ok(())
+        })
+        .map_err(|e| e.into())
+    }
+
     fn lock_write(&self) -> Result<Lock, Error> {
         Python::with_gil(|py| {
             let lock = self.to_object(py).call_method0(py, "lock_write")?;
@@ -449,6 +479,8 @@ pub trait MutableTree: Tree {
         })
         .map_err(|e| e.into())
     }
+
+    fn as_tree(&self) -> &dyn Tree;
 }
 
 pub struct RevisionTree(pub PyObject);
@@ -496,233 +528,13 @@ impl RevisionTree {
     }
 }
 
-pub struct WorkingTree(pub PyObject);
-
-impl Clone for WorkingTree {
-    fn clone(&self) -> Self {
-        Python::with_gil(|py| WorkingTree(self.0.clone_ref(py)))
-    }
-}
-
-impl ToPyObject for WorkingTree {
-    fn to_object(&self, py: Python) -> PyObject {
-        self.0.to_object(py)
-    }
-}
-
-impl WorkingTree {
-    pub fn is_control_filename(&self, path: &Path) -> bool {
-        Python::with_gil(|py| {
-            self.to_object(py)
-                .call_method1(py, "is_control_filename", (path,))
-                .unwrap()
-                .extract(py)
-                .unwrap()
-        })
-    }
-
-    /// Return the base path for this working tree.
-    pub fn basedir(&self) -> PathBuf {
-        Python::with_gil(|py| {
-            self.to_object(py)
-                .getattr(py, "basedir")
-                .unwrap()
-                .extract(py)
-                .unwrap()
-        })
-    }
-
-    /// Return the branch for this working tree.
-    pub fn branch(&self) -> Box<dyn Branch> {
-        Python::with_gil(|py| {
-            let branch = self.to_object(py).getattr(py, "branch").unwrap();
-            Box::new(RegularBranch::new(branch)) as Box<dyn Branch>
-        })
-    }
-
-    /// Return the control directory for this working tree.
-    pub fn controldir(&self) -> ControlDir {
-        Python::with_gil(|py| {
-            let controldir = self.to_object(py).getattr(py, "controldir").unwrap();
-            ControlDir::new(controldir)
-        })
-    }
-
-    pub fn open(path: &Path) -> Result<WorkingTree, Error> {
-        Python::with_gil(|py| {
-            let m = py.import_bound("breezy.workingtree")?;
-            let c = m.getattr("WorkingTree")?;
-            let wt = c.call_method1("open", (path,))?;
-            Ok(WorkingTree(wt.to_object(py)))
-        })
-    }
-
-    pub fn open_containing(path: &Path) -> Result<(WorkingTree, PathBuf), Error> {
-        Python::with_gil(|py| {
-            let m = py.import_bound("breezy.workingtree")?;
-            let c = m.getattr("WorkingTree")?;
-            let (wt, p): (Bound<PyAny>, String) =
-                c.call_method1("open_containing", (path,))?.extract()?;
-            Ok((WorkingTree(wt.to_object(py)), PathBuf::from(p)))
-        })
-    }
-
-    pub fn basis_tree(&self) -> Result<crate::tree::RevisionTree, Error> {
-        Python::with_gil(|py| {
-            let tree = self.to_object(py).call_method0(py, "basis_tree")?;
-            Ok(RevisionTree(tree))
-        })
-    }
-
-    pub fn revision_tree(&self, revision_id: &RevisionId) -> Result<Box<RevisionTree>, Error> {
-        Python::with_gil(|py| {
-            let tree = self.to_object(py).call_method1(
-                py,
-                "revision_tree",
-                (revision_id.to_object(py),),
-            )?;
-            Ok(Box::new(RevisionTree(tree)))
-        })
-    }
-
-    pub fn get_tag_dict(&self) -> Result<std::collections::HashMap<String, RevisionId>, Error> {
-        Python::with_gil(|py| {
-            let branch = self.to_object(py).getattr(py, "branch")?;
-            let tags = branch.getattr(py, "tags")?;
-            let tag_dict = tags.call_method0(py, "get_tag_dict")?;
-            tag_dict.extract(py)
-        })
-        .map_err(|e: PyErr| -> Error { e.into() })
-    }
-
-    pub fn abspath(&self, path: &Path) -> Result<PathBuf, Error> {
-        Python::with_gil(|py| {
-            Ok(self
-                .to_object(py)
-                .call_method1(py, "abspath", (path,))?
-                .extract(py)?)
-        })
-    }
-
-    pub fn relpath(&self, path: &Path) -> Result<PathBuf, Error> {
-        Python::with_gil(|py| {
-            Ok(self
-                .to_object(py)
-                .call_method1(py, "relpath", (path,))?
-                .extract(py)?)
-        })
-    }
-
-    pub fn supports_setting_file_ids(&self) -> bool {
-        Python::with_gil(|py| {
-            self.to_object(py)
-                .call_method0(py, "supports_setting_file_ids")
-                .unwrap()
-                .extract(py)
-                .unwrap()
-        })
-    }
-
-    pub fn add(&self, paths: &[&Path]) -> Result<(), Error> {
-        Python::with_gil(|py| {
-            self.to_object(py)
-                .call_method1(py, "add", (paths.to_vec(),))
-        })
-        .map_err(|e| e.into())
-        .map(|_| ())
-    }
-
-    pub fn smart_add(&self, paths: &[&Path]) -> Result<(), Error> {
-        Python::with_gil(|py| {
-            self.to_object(py)
-                .call_method1(py, "smart_add", (paths.to_vec(),))
-        })
-        .map_err(|e| e.into())
-        .map(|_| ())
-    }
-
-    pub fn commit(
-        &self,
-        message: &str,
-        allow_pointless: Option<bool>,
-        committer: Option<&str>,
-        specific_files: Option<&[&Path]>,
-    ) -> Result<RevisionId, Error> {
-        Python::with_gil(|py| {
-            let kwargs = pyo3::types::PyDict::new_bound(py);
-            if let Some(committer) = committer {
-                kwargs.set_item("committer", committer).unwrap();
-            }
-            if let Some(specific_files) = specific_files {
-                kwargs.set_item("specific_files", specific_files).unwrap();
-            }
-            if let Some(allow_pointless) = allow_pointless {
-                kwargs.set_item("allow_pointless", allow_pointless).unwrap();
-            }
-
-            let null_commit_reporter = py
-                .import_bound("breezy.commit")
-                .unwrap()
-                .getattr("NullCommitReporter")
-                .unwrap()
-                .call0()
-                .unwrap();
-            kwargs.set_item("reporter", null_commit_reporter).unwrap();
-
-            Ok(self
-                .to_object(py)
-                .call_method_bound(py, "commit", (message,), Some(&kwargs))
-                .map_err(|e| -> Error { e.into() })?
-                .extract(py)
-                .unwrap())
-        })
-    }
-
-    pub fn last_revision(&self) -> Result<RevisionId, Error> {
-        Python::with_gil(|py| {
-            let last_revision = self.to_object(py).call_method0(py, "last_revision")?;
-            Ok(RevisionId::from(last_revision.extract::<Vec<u8>>(py)?))
-        })
-    }
-
-    pub fn pull(
-        &self,
-        source: &dyn crate::branch::Branch,
-        overwrite: Option<bool>,
-    ) -> Result<(), Error> {
-        Python::with_gil(|py| {
-            let kwargs = {
-                let kwargs = pyo3::types::PyDict::new_bound(py);
-                if let Some(overwrite) = overwrite {
-                    kwargs.set_item("overwrite", overwrite).unwrap();
-                }
-                kwargs
-            };
-            self.to_object(py)
-                .call_method_bound(py, "pull", (source.to_object(py),), Some(&kwargs))
-        })
-        .map_err(|e| e.into())
-        .map(|_| ())
-    }
-}
-
-impl From<PyObject> for WorkingTree {
-    fn from(obj: PyObject) -> Self {
-        WorkingTree(obj)
-    }
-}
-
-impl Tree for WorkingTree {}
-
-impl MutableTree for WorkingTree {}
-
 #[derive(Debug, PartialEq, Eq, Clone)]
 pub struct TreeChange {
     pub path: (Option<PathBuf>, Option<PathBuf>),
     pub changed_content: bool,
     pub versioned: (Option<bool>, Option<bool>),
     pub name: (Option<std::ffi::OsString>, Option<std::ffi::OsString>),
-    pub kind: (Option<String>, Option<String>),
+    pub kind: (Option<Kind>, Option<Kind>),
     pub executable: (Option<bool>, Option<bool>),
     pub copied: bool,
 }
@@ -806,7 +618,13 @@ impl From<&dyn Branch> for MemoryTree {
 
 impl Tree for MemoryTree {}
 
-impl MutableTree for MemoryTree {}
+impl MutableTree for MemoryTree {
+    fn as_tree(&self) -> &dyn Tree {
+        self
+    }
+}
+
+pub use crate::workingtree::WorkingTree;
 
 #[cfg(test)]
 mod tests {
@@ -820,7 +638,11 @@ mod tests {
         let path = td.path().join("foo");
         std::fs::write(&path, b"").unwrap();
         wt.add(&[(std::path::Path::new("foo"))]).unwrap();
-        wt.commit("Initial commit", None, None, None).unwrap();
+        wt.build_commit()
+            .message("Initial commit")
+            .reporter(&crate::commit::NullCommitReporter::new())
+            .commit()
+            .unwrap();
         assert!(wt.has_filename(&path));
         wt.remove(&[&path]).unwrap();
         assert!(!wt.is_versioned(&path));
